@@ -75,7 +75,6 @@ struct _SnItem
 
   gchar               *bus_name;
   gchar               *object_path;
-  gchar               *service;
   gchar               *key;
 
   gchar               *id;
@@ -107,7 +106,6 @@ enum
   PROP_0,
   PROP_BUS_NAME,
   PROP_OBJECT_PATH,
-  PROP_SERVICE,
   PROP_KEY,
   PROP_EXPOSED
 };
@@ -178,12 +176,6 @@ sn_item_class_init (SnItemClass *klass)
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (object_class,
-                                   PROP_SERVICE,
-                                   g_param_spec_string ("service", NULL, NULL, NULL,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_STRINGS));
-
   g_object_class_install_property (object_class,
                                    PROP_KEY,
                                    g_param_spec_string ("key", NULL, NULL, NULL,
@@ -260,7 +252,6 @@ sn_item_init (SnItem *item)
 
   item->bus_name = NULL;
   item->object_path = NULL;
-  item->service = NULL;
   item->key = NULL;
 
   item->id = NULL;
@@ -302,7 +293,6 @@ sn_item_finalize (GObject *object)
 
   g_free (item->bus_name);
   g_free (item->object_path);
-  g_free (item->service);
   g_free (item->key);
 
   g_free (item->id);
@@ -343,10 +333,6 @@ sn_item_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_SERVICE:
-      g_value_set_string (value, item->service);
-      break;
-
     case PROP_KEY:
       g_value_set_string (value, item->key);
       break;
@@ -383,11 +369,6 @@ sn_item_set_property (GObject      *object,
       item->object_path = g_value_dup_string (value);
       break;
 
-    case PROP_SERVICE:
-      g_free (item->service);
-      item->service = g_value_dup_string (value);
-      break;
-
     case PROP_KEY:
       g_free (item->key);
       item->key = g_value_dup_string (value);
@@ -422,11 +403,15 @@ sn_item_name_owner_changed (GDBusConnection *connection,
                             GVariant        *parameters,
                             gpointer         user_data)
 {
-  SnItem *item = user_data;
-  gchar  *new_owner;
+  SnItem   *item = user_data;
+  gchar    *new_owner;
+  gboolean  finish;
 
   g_variant_get (parameters, "(sss)", NULL, NULL, &new_owner);
-  return_and_finish_if_true (new_owner == NULL || strlen (new_owner) == 0);
+  finish = new_owner == NULL || strlen (new_owner) == 0;
+  g_free (new_owner);
+
+  return_and_finish_if_true (finish);
 }
 
 
@@ -580,6 +565,8 @@ sn_item_signal_received (GDBusProxy *proxy,
     {
       g_variant_get (parameters, "(s)", &status);
       exposed = sn_item_status_is_exposed (status);
+      g_free (status);
+
       if (exposed != item->exposed)
         {
           item->exposed = exposed;
@@ -597,9 +584,9 @@ sn_item_pixbuf_equals (GdkPixbuf *p1,
 {
   guchar *p1p, *p2p;
   guint   p1l, p2l;
-  gint    i;
+  guint   i;
 
-  if (p1 == p2 || p1 == NULL && p2 == NULL)
+  if (p1 == p2 || (p1 == NULL && p2 == NULL))
     return TRUE;
 
   if ((p1 == NULL) != (p2 == NULL))
@@ -611,7 +598,7 @@ sn_item_pixbuf_equals (GdkPixbuf *p1,
   if (p1l != p2l)
     return FALSE;
 
-  for (gint i = 0; i < p1l; i++)
+  for (i = 0; i < p1l; i++)
     if (p1p[i] != p2p[i])
       return FALSE;
 
@@ -648,7 +635,7 @@ sn_item_extract_pixbuf (GVariant *variant)
         {
           size = g_variant_get_size (array_value);
           /* sanity check */
-          if (size == 4 * width * height)
+          if (size == (gsize)(4 * width * height))
             {
               /* find the largest image */
               data = g_variant_get_data (array_value);
@@ -664,6 +651,8 @@ sn_item_extract_pixbuf (GVariant *variant)
         }
     }
 
+  g_variant_iter_free (iter);
+
   if (array != NULL)
     {
       /* argb to rgba */
@@ -698,8 +687,9 @@ sn_item_get_all_properties_result (GObject      *source_object,
   const gchar  *name;
   GVariant     *value;
 
-  const gchar  *str_val1;
-  const gchar  *str_val2;
+  const gchar  *cstr_val1;
+  gchar        *str_val1;
+  gchar        *str_val2;
   gboolean      bool_val1;
   GdkPixbuf    *pb_val1;
 
@@ -746,8 +736,8 @@ sn_item_get_all_properties_result (GObject      *source_object,
       }
     else if (!g_strcmp0 (name, "Status"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        bool_val1 = sn_item_status_is_exposed (str_val1);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        bool_val1 = sn_item_status_is_exposed (cstr_val1);
         if (bool_val1 != item->exposed)
           {
             item->exposed = bool_val1;
@@ -756,14 +746,16 @@ sn_item_get_all_properties_result (GObject      *source_object,
       }
     else if (!g_strcmp0 (name, "Title"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, title, update_tooltip);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, title, update_tooltip);
       }
     else if (!g_strcmp0 (name, "ToolTip"))
       {
         g_variant_get (value, "(sa(iiay)ss)", NULL, NULL, &str_val1, &str_val2);
         update_new_string (str_val1, tooltip_title, update_tooltip);
         update_new_string (str_val2, tooltip_subtitle, update_tooltip);
+        g_free (str_val1);
+        g_free (str_val2);
       }
     else if (!g_strcmp0 (name, "ItemIsMenu"))
       {
@@ -776,18 +768,18 @@ sn_item_get_all_properties_result (GObject      *source_object,
       }
     else if (!g_strcmp0 (name, "Menu"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, menu_object_path, update_menu);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, menu_object_path, update_menu);
       }
     else if (!g_strcmp0 (name, "IconThemePath"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, icon_theme_path, update_icon);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, icon_theme_path, update_icon);
       }
     else if (!g_strcmp0 (name, "IconName"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, icon_name, update_icon);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, icon_name, update_icon);
       }
     else if (!g_strcmp0 (name, "IconPixmap"))
       {
@@ -796,13 +788,13 @@ sn_item_get_all_properties_result (GObject      *source_object,
       }
     else if (!g_strcmp0 (name, "IconAccessibleDesc"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, icon_desc, update_icon);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, icon_desc, update_tooltip);
       }
     else if (!g_strcmp0 (name, "AttentionIconName"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, attention_icon_name, update_icon);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, attention_icon_name, update_icon);
       }
     else if (!g_strcmp0 (name, "AttentionIconPixmap"))
       {
@@ -811,13 +803,13 @@ sn_item_get_all_properties_result (GObject      *source_object,
       }
     else if (!g_strcmp0 (name, "AttentionAccessibleDesc"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, attention_desc, update_icon);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, attention_desc, update_tooltip);
       }
     else if (!g_strcmp0 (name, "OverlayIconName"))
       {
-        str_val1 = g_variant_get_string (value, NULL);
-        update_new_string (str_val1, overlay_icon_name, update_icon);
+        cstr_val1 = g_variant_get_string (value, NULL);
+        update_new_string (cstr_val1, overlay_icon_name, update_icon);
       }
     else if (!g_strcmp0 (name, "OverlayIconPixmap"))
       {
@@ -929,10 +921,12 @@ sn_item_get_tooltip (SnItem       *item,
   if (subtitle == NULL)
     subtitle = (gpointer)&stub;
 
+  #define sn_subtitle(subtitle) (g_strcmp0 (subtitle, *title) ? subtitle : NULL)
+
   if (item->tooltip_title != NULL && item->tooltip_subtitle != NULL)
     {
       *title = item->tooltip_title;
-      *subtitle = item->tooltip_subtitle;
+      *subtitle = sn_subtitle (item->tooltip_subtitle);
     }
   else if (item->attention_desc != NULL)
     {
@@ -940,12 +934,12 @@ sn_item_get_tooltip (SnItem       *item,
       if (item->tooltip_title != NULL)
         {
           *title = item->tooltip_title;
-          *subtitle = item->attention_desc;
+          *subtitle = sn_subtitle (item->attention_desc);
         }
       else if (item->title != NULL)
         {
           *title = item->title;
-          *subtitle = item->attention_desc;
+          *subtitle = sn_subtitle (item->attention_desc);
         }
       else
         {
@@ -959,12 +953,12 @@ sn_item_get_tooltip (SnItem       *item,
       if (item->tooltip_title != NULL)
         {
           *title = item->tooltip_title;
-          *subtitle = item->icon_desc;
+          *subtitle = sn_subtitle (item->icon_desc);
         }
       else if (item->title != NULL)
         {
           *title = item->title;
-          *subtitle = item->icon_desc;
+          *subtitle = sn_subtitle (item->icon_desc);
         }
       else
         {
@@ -988,7 +982,7 @@ sn_item_get_tooltip (SnItem       *item,
       *subtitle = NULL;
     }
 
-  #undef sn_non_empty
+  #undef sn_subtitle
 }
 
 
